@@ -4,10 +4,10 @@ use std::path::PathBuf;
 use std::{fs, io, process};
 
 use clap::{arg, value_parser, ArgAction, ArgMatches, Command};
-use yara_x_fmt::Formatter;
+use yara_x_fmt::{Formatter, Indentation};
 
 use crate::config::Config;
-use crate::help::FMT_CHECK_MODE;
+use crate::help;
 
 pub fn fmt() -> Command {
     super::command("fmt")
@@ -19,19 +19,35 @@ pub fn fmt() -> Command {
                 .value_parser(value_parser!(PathBuf))
                 .action(ArgAction::Append),
         )
-        .arg(arg!(-c --check  "Run in 'check' mode").long_help(FMT_CHECK_MODE))
+        .arg(
+            arg!(-c --check  "Run in 'check' mode")
+                .long_help(help::FMT_CHECK_MODE),
+        )
+        .arg(
+            arg!(-t - -"tab-size" <NUM_SPACES>)
+                .help("Tab size (in spaces) used in source files")
+                .long_help(help::FMT_TAB_SIZE)
+                .default_value("4")
+                .value_parser(value_parser!(usize)),
+        )
 }
 
 pub fn exec_fmt(args: &ArgMatches, config: &Config) -> anyhow::Result<()> {
     let files = args.get_many::<PathBuf>("FILE").unwrap();
     let check = args.get_flag("check");
+    let tab_size = args.get_one::<usize>("tab-size").unwrap();
 
     let formatter = Formatter::new()
+        .input_tab_size(*tab_size)
         .align_metadata(config.fmt.meta.align_values)
         .align_patterns(config.fmt.patterns.align_values)
         .indent_section_headers(config.fmt.rule.indent_section_headers)
         .indent_section_contents(config.fmt.rule.indent_section_contents)
-        .indent_spaces(config.fmt.rule.indent_spaces)
+        .indentation(if config.fmt.rule.indent_spaces == 0 {
+            Indentation::Tabs
+        } else {
+            Indentation::Spaces(config.fmt.rule.indent_spaces as usize)
+        })
         .newline_before_curly_brace(config.fmt.rule.newline_before_curly_brace)
         .empty_line_before_section_header(
             config.fmt.rule.empty_line_before_section_header,
