@@ -1,4 +1,4 @@
-use assert_cmd::Command;
+use assert_cmd::{cargo_bin, Command};
 use assert_fs::prelude::*;
 use assert_fs::TempDir;
 use predicates::prelude::*;
@@ -26,14 +26,13 @@ fn metadata() {
         )
         .unwrap();
 
-    Command::cargo_bin("yr")
-        .unwrap()
+    Command::new(cargo_bin!("yr"))
         .arg("--config")
         .arg(config_file.path())
         .arg("check")
         .arg("src/tests/testdata/foo.yar")
         .assert()
-        .success()
+        .code(2)
         .stdout("[ WARN ] src/tests/testdata/foo.yar\n")
         .stderr(
             r#"warning[missing_metadata]: required metadata is missing
@@ -72,14 +71,13 @@ warning[text_as_hex]: hex pattern could be written as text literal
         )
         .unwrap();
 
-    Command::cargo_bin("yr")
-        .unwrap()
+    Command::new(cargo_bin!("yr"))
         .arg("--config")
         .arg(config_file.path())
         .arg("check")
         .arg(yar_file.path())
         .assert()
-        .success()
+        .code(2)
         .stderr(predicate::str::contains(
             "warning[invalid_metadata]: metadata `md5` is not valid",
         ))
@@ -130,14 +128,14 @@ fn check_rule_name_warning() {
         )
         .unwrap();
 
-    Command::cargo_bin("yr")
-        .unwrap()
+    Command::new(cargo_bin!("yr"))
         .arg("--config")
         .arg(config_file.path())
         .arg("check")
         .arg("src/tests/testdata/foo.yar")
         .assert()
-        .success()
+        .failure()
+        .code(2)
         .stdout("[ WARN ] src/tests/testdata/foo.yar\n");
 }
 
@@ -156,14 +154,14 @@ fn check_rule_name_error() {
         )
         .unwrap();
 
-    Command::cargo_bin("yr")
-        .unwrap()
+    Command::new(cargo_bin!("yr"))
         .arg("--config")
         .arg(config_file.path())
         .arg("check")
         .arg("src/tests/testdata/foo.yar")
         .assert()
-        .success()
+        .failure()
+        .code(1)
         .stdout(
             r#"[ FAIL ] src/tests/testdata/foo.yar
 error[E039]: rule name does not match regex `APT_.+`
@@ -189,8 +187,7 @@ fn config_error() {
         )
         .unwrap();
 
-    Command::cargo_bin("yr")
-        .unwrap()
+    Command::new(cargo_bin!("yr"))
         .arg("--config")
         .arg(config_file.path())
         .arg("check")
