@@ -11,7 +11,25 @@ use yara_x_macros::ErrorEnum;
 use yara_x_macros::ErrorStruct;
 use yara_x_parser::ast;
 
-use crate::compiler::report::{Level, Report, ReportBuilder, CodeLoc, Label, Footer};
+use crate::compiler::report::{Level, Patch, Report, ReportBuilder, CodeLoc, Label, Footer};
+
+/// Error returned by [`crate::Compiler::emit_wasm_file`].
+#[derive(Error, Debug)]
+#[error(transparent)]
+#[doc(hidden)]
+pub struct EmitWasmError(#[from] anyhow::Error);
+
+/// Error returned by [`crate::Compiler::switch_warning`] when the warning
+/// code is not valid.
+#[derive(Error, Debug, Eq, PartialEq)]
+#[error("`{0}` is not a valid warning code")]
+pub struct InvalidWarningCode(String);
+
+impl InvalidWarningCode {
+    pub(crate) fn new(code: String) -> Self {
+        Self(code)
+    }
+}
 
 /// Error returned while serializing/deserializing compiled rules.
 #[derive(Error, Debug)]
@@ -37,12 +55,6 @@ pub enum SerializationError {
     InvalidWASM(#[from] anyhow::Error),
 }
 
-/// Error returned by [`crate::Compiler::emit_wasm_file`].
-#[derive(Error, Debug)]
-#[error(transparent)]
-#[doc(hidden)]
-pub struct EmitWasmError(#[from] anyhow::Error);
-
 /// Error returned when rule compilation fails.
 #[allow(missing_docs)]
 #[non_exhaustive]
