@@ -38,3 +38,25 @@ fn e2e_test() {
 
     assert_eq!(results.matching_rules().len(), 1);
 }
+
+// Test case that covers the following issue:
+// https://github.com/robo9k/rust-magic/issues/358
+#[test]
+fn magic_crash() {
+    let rules = crate::compile(
+        r#"
+    import "magic"
+    rule t {
+      condition:
+        magic.mime_type() == "foo"
+    }"#,
+    )
+    .unwrap();
+
+    let mut scanner = crate::scanner::Scanner::new(&rules);
+    let results = scanner
+        .scan(&[0x00, 0x00, 0xfe, 0xff, 0xff, 0xff, 0x2a, 0x08])
+        .unwrap();
+
+    assert_eq!(results.matching_rules().len(), 0);
+}
