@@ -3,9 +3,9 @@ use crate::modules::protos::time::*;
 use std::time::{SystemTime, UNIX_EPOCH};
 
 #[module_main]
-fn main(_data: &[u8], _meta: Option<&[u8]>) -> Time {
+fn main(_data: &[u8], _meta: Option<&[u8]>) -> Result<Time, ModuleError> {
     // Nothing to do, but we have to return our protobuf
-    Time::new()
+    Ok(Time::new())
 }
 
 #[module_export]
@@ -22,7 +22,7 @@ mod tests {
     fn now() {
         rule_true!(
             r#"
-            import "time" 
+            import "time"
             rule test { condition: time.now() >= 0 }"#,
             &[]
         );
