@@ -18,7 +18,6 @@ use bitvec::order::Lsb0;
 use bitvec::slice::BitSlice;
 use bstr::{BString, ByteSlice};
 use indexmap::IndexMap;
-use memx::memeq;
 use protobuf::{MessageDyn, MessageFull};
 use regex_automata::meta::Regex;
 use rustc_hash::{FxHashMap, FxHashSet};
@@ -48,6 +47,7 @@ use crate::{wasm, Variable};
 /// Represents the states in which a scanner can be.
 pub(crate) enum ScanState<'a> {
     Idle,
+    Timeout,
     ScanningData(ScannedData<'a>),
     ScanningBlock((usize, &'a [u8])),
     Finished(DataSnippets<'a>),
@@ -182,8 +182,9 @@ impl ScanContext<'_, '_> {
             self.time_spent_in_rule.iter(),
         ) {
             let mut pattern_matching_time = 0;
-            for (_, _, pattern_id, _) in rule.patterns.iter() {
-                if let Some(d) = self.time_spent_in_pattern.get(pattern_id) {
+            for p in rule.patterns.iter() {
+                if let Some(d) = self.time_spent_in_pattern.get(&p.pattern_id)
+                {
                     pattern_matching_time += *d;
                 }
             }
@@ -356,6 +357,11 @@ impl ScanContext<'_, '_> {
         obj_ref
     }
 
+    /// Get the value of the global variable `filesize`.
+    pub(crate) fn get_filesize(&mut self) -> i64 {
+        self.wasm_filesize.unwrap().get(self.wasm_store_mut()).i64().unwrap()
+    }
+
     /// Set the value of the global variable `filesize`.
     pub(crate) fn set_filesize(&mut self, filesize: i64) {
         self.wasm_filesize
@@ -374,8 +380,8 @@ impl ScanContext<'_, '_> {
     /// calls ScanContext::search_for_patterns (which does the Aho-Corasick
     /// scanning) only if necessary.
     ///
-    /// This will return `Err(ScanError::Timeout)`, when the scan timeout is
-    /// reached while WASM code is being executed.
+    /// This will return [ScanError::Timeout], if a timeout occurs while
+    /// searching for patterns or evaluating the conditions.
     pub(crate) fn eval_conditions(&mut self) -> Result<(), ScanError> {
         // Save the time in which the evaluation started.
         #[cfg(feature = "rules-profiling")]
@@ -448,8 +454,16 @@ impl ScanContext<'_, '_> {
             }
         }
 
+        // The WASM code that evaluates the conditions returns
+        // `ScanError::Timeout` if a timeout occurs during its execution.
+        // However, a timeout may also happen while `search_for_patterns`
+        // is running. In that case, the function returns `Ok(0)` but the
+        // scan state is updated to `ScanState::Timeout`.
         match eval_result {
-            Ok(0) => Ok(()),
+            Ok(0) => match self.scan_state {
+                ScanState::Timeout => Err(ScanError::Timeout),
+                _ => Ok(()),
+            },
             Ok(v) => panic!("WASM main returned: {v}"),
             Err(err) if err.is::<ScanError>() => {
                 Err(err.downcast::<ScanError>().unwrap())
@@ -486,6 +500,9 @@ impl ScanContext<'_, '_> {
         // to some struct.
         self.current_struct = None;
 
+        // Clear module outputs from previous scans.
+        self.module_outputs.clear();
+
         // Move the matching rules to the `matching_rules` vector, leaving the
         // `matching_rules_per_ns` map empty.
         for rules in self.matching_rules_per_ns.values_mut() {
@@ -717,47 +734,25 @@ impl ScanContext<'_, '_> {
         }
     }
 
-    /// Search for patterns in the scanned data.
-    ///
-    /// The pattern search phase is when YARA scans the data looking for the
-    /// patterns declared in rules. All the patterns are searched simultaneously
-    /// using the Aho-Corasick algorithm. This phase is triggered lazily during
-    /// the evaluation of the rule conditions, when some of the conditions need
-    /// to know if a pattern matched or not.
-    ///
-    /// This function won't be called if the conditions can be fully evaluated
-    /// without looking for any of the patterns. If it must be called, it will be
-    /// called only once.
-    pub(crate) fn search_for_patterns(&mut self) -> Result<(), ScanError> {
-        let ac = self.compiled_rules.ac_automaton();
-
+    /// The Aho-Corasick search loop.
+    pub(crate) fn ac_search_loop(
+        &mut self,
+        base: usize,
+        data: &[u8],
+        block_scanning_mode: bool,
+    ) -> Result<(), ScanError> {
         let mut vm = VM {
             pike_vm: PikeVM::new(self.compiled_rules.re_code()),
             fast_vm: FastVM::new(self.compiled_rules.re_code()),
         };
 
+        let ac = self.compiled_rules.ac_automaton();
         let atoms = self.compiled_rules.atoms();
-
-        #[cfg(any(feature = "rules-profiling", feature = "logging"))]
-        let scan_start = self.clock.raw();
+        let filesize = self.get_filesize();
 
         #[cfg(feature = "logging")]
         let mut atom_matches = 0_usize;
 
-        // Take ownership of the scan state, while searching for
-        // the patterns, `self.scan_state` is left as `Idle`.
-        let state = self.scan_state.take();
-
-        let (base, data) = match &state {
-            ScanState::ScanningData(data) => (0, data.as_ref()),
-            ScanState::ScanningBlock((base, data)) => (*base, *data),
-            _ => panic!(),
-        };
-
-        // Verify the anchored pattern first. These are patterns that can
-        // match at a single known offset within the data.
-        self.verify_anchored_patterns(base, data);
-
         for ac_match in ac.find_overlapping_iter(data) {
             #[cfg(feature = "logging")]
             {
@@ -765,11 +760,6 @@ impl ScanContext<'_, '_> {
             }
 
             if HEARTBEAT_COUNTER.load(Ordering::Relaxed) >= self.deadline {
-                #[cfg(feature = "logging")]
-                log::info!(
-                    "Scan timeout after: {:?}",
-                    self.clock.delta(scan_start, self.clock.raw())
-                );
                 return Err(ScanError::Timeout);
             }
 
@@ -796,13 +786,26 @@ impl ScanContext<'_, '_> {
 
             // Check if the potentially matching pattern has reached the
             // maximum number of allowed matches. In that case continue without
-            // verifying the match. `get_unchecked` is used for performance
-            // reasons, the number of bits in the bit vector is guaranteed to
-            // be the number of patterns.
+            // verifying the match.
             if self.limit_reached.contains(pattern_id) {
                 continue;
             }
 
+            // If there are file size bounds associated to the pattern, but
+            // the currently scanned file does not satisfy them, no further
+            // confirmation is needed. The rule won't match regardless of
+            // whether the pattern matches or not. This is not done in block
+            // scanning mode as `filesize` is undefined in that mode.
+            if !block_scanning_mode {
+                if let Some(bounds) =
+                    self.compiled_rules.filesize_bounds(*pattern_id)
+                {
+                    if !bounds.contains(filesize) {
+                        continue;
+                    }
+                }
+            }
+
             #[cfg(feature = "rules-profiling")]
             let verification_start = self.clock.raw();
 
@@ -978,18 +981,60 @@ impl ScanContext<'_, '_> {
             }
         }
 
+        #[cfg(feature = "logging")]
+        log::info!("Atom matches: {}", atom_matches);
+
+        Ok(())
+    }
+
+    /// Search for patterns in the scanned data.
+    ///
+    /// The pattern search phase is when YARA scans the data looking for the
+    /// patterns declared in rules. All the patterns are searched simultaneously
+    /// using the Aho-Corasick algorithm. This phase is triggered lazily during
+    /// the evaluation of the rule conditions, when some of the conditions need
+    /// to know if a pattern matched or not.
+    ///
+    /// This function won't be called if the conditions can be fully evaluated
+    /// without looking for any of the patterns. If it must be called, it will be
+    /// called only once.
+    ///
+    /// In case of timeout, this function returns [ScanError::Timeout] and sets
+    /// the scan state to [ScanState::Timeout].
+    pub(crate) fn search_for_patterns(&mut self) -> Result<(), ScanError> {
+        // Take ownership of the scan state, while searching for
+        // the patterns, `self.scan_state` is left as `Idle`.
+        let state = self.scan_state.take();
+
+        let (base, data, block_scanning_mode) = match &state {
+            ScanState::ScanningData(data) => (0, data.as_ref(), false),
+            ScanState::ScanningBlock((base, data)) => (*base, *data, true),
+            _ => panic!(),
+        };
+
+        #[cfg(any(feature = "rules-profiling", feature = "logging"))]
+        let scan_start = self.clock.raw();
+
+        // Verify the anchored pattern first. These are patterns that can
+        // match at a single known offset within the data.
+        self.verify_anchored_patterns(base, data);
+
+        let result = match self.ac_search_loop(base, data, block_scanning_mode)
+        {
+            Ok(_) => {
+                self.scan_state = state;
+                Ok(())
+            }
+            Err(ScanError::Timeout) => {
+                self.scan_state = ScanState::Timeout;
+                Err(ScanError::Timeout)
+            }
+            _ => unreachable!(),
+        };
+
         #[cfg(any(feature = "rules-profiling", feature = "logging"))]
         let scan_end = self.clock.raw();
 
-        #[cfg(feature = "logging")]
-        {
-            log::info!(
-                "Scan time: {:?}",
-                self.clock.delta(scan_start, scan_end)
-            );
-            log::info!("Atom matches: {}", atom_matches);
-        }
-
         // Adjust the rule evaluation start time to exclude the time spent
         // searching for patterns. Since the `search_for_pattern` function
         // is invoked lazily during the evaluation of some rule, the overall
@@ -1002,10 +1047,15 @@ impl ScanContext<'_, '_> {
                 scan_end.saturating_sub(scan_start);
         }
 
-        // Bring back ownership of the scanned to the ScanContext.
-        self.scan_state = state;
+        #[cfg(feature = "logging")]
+        {
+            log::info!(
+                "Scan time: {:?}",
+                self.clock.delta(scan_start, scan_end)
+            );
+        }
 
-        Ok(())
+        result
     }
 
     fn verify_anchored_patterns(&mut self, base: usize, data: &[u8]) {
@@ -1488,7 +1538,7 @@ fn verify_xor_match(
     }
 
     if key == 0 {
-        if !memeq(pattern, &scanned_data[match_start..match_end]) {
+        if pattern != &scanned_data[match_start..match_end] {
             return None;
         }
     } else {
