@@ -34,7 +34,7 @@ impl Map {
     #[allow(clippy::declare_interior_mutable_const)]
     const BUILTIN_METHODS: OnceCell<Rc<SymbolTable>> = OnceCell::new();
 
-    pub fn builtin_methods(&self) -> Rc<SymbolTable> {
+    pub fn builtin_methods() -> Rc<SymbolTable> {
         #[allow(clippy::borrow_interior_mutable_const)]
         Self::BUILTIN_METHODS
             .get_or_init(|| {
