@@ -39,7 +39,7 @@ use pyo3::{create_exception, IntoPyObjectExt};
 use strum_macros::{Display, EnumString};
 
 use ::yara_x as yrx;
-use ::yara_x::mods::*;
+
 use yara_x_fmt::Indentation;
 
 fn dict_to_json(dict: Bound<PyAny>) -> PyResult<serde_json::Value> {
@@ -57,10 +57,15 @@ fn dict_to_json(dict: Bound<PyAny>) -> PyResult<serde_json::Value> {
 #[derive(Debug, Clone, Display, EnumString, PartialEq)]
 #[strum(ascii_case_insensitive)]
 enum SupportedModules {
+    #[cfg(feature = "lnk-module")]
     Lnk,
+    #[cfg(feature = "macho-module")]
     Macho,
+    #[cfg(feature = "elf-module")]
     Elf,
+    #[cfg(feature = "pe-module")]
     Pe,
+    #[cfg(feature = "dotnet-module")]
     Dotnet,
 }
 
@@ -197,11 +202,11 @@ impl Read for PyReader {
                 self.obj.call_method1(py, consts::read(py), (buf.len(),))?;
 
             if self.is_text_io {
-                let bytes = data.extract::<Cow<str>>(py)?;
+                let bytes = data.extract::<Cow<str>>(py).unwrap();
                 buf.write_all(bytes.as_bytes())?;
                 Ok(bytes.len())
             } else {
-                let bytes = data.extract::<Cow<[u8]>>(py)?;
+                let bytes = data.extract::<Cow<[u8]>>(py).unwrap();
                 buf.write_all(bytes.as_ref())?;
                 Ok(bytes.len())
             }
@@ -281,18 +286,36 @@ impl Module {
     /// Invoke the module with provided data.
     ///
     /// Returns None if the module didn't produce any output for the given data.
+    #[allow(unreachable_code, unused_variables, unreachable_patterns)]
     fn invoke<'py>(
         &'py self,
         py: Python<'py>,
         data: &[u8],
     ) -> PyResult<Bound<'py, PyAny>> {
-        let module_output = match self._module {
-            SupportedModules::Macho => invoke_dyn::<Macho>(data),
-            SupportedModules::Lnk => invoke_dyn::<Lnk>(data),
-            SupportedModules::Elf => invoke_dyn::<ELF>(data),
-            SupportedModules::Pe => invoke_dyn::<PE>(data),
-            SupportedModules::Dotnet => invoke_dyn::<Dotnet>(data),
-        };
+        let module_output: Option<Box<dyn protobuf::MessageDyn>> =
+            match self._module {
+                #[cfg(feature = "macho-module")]
+                SupportedModules::Macho => {
+                    yrx::mods::invoke_dyn::<yrx::mods::Macho>(data)
+                }
+                #[cfg(feature = "lnk-module")]
+                SupportedModules::Lnk => {
+                    yrx::mods::invoke_dyn::<yrx::mods::Lnk>(data)
+                }
+                #[cfg(feature = "elf-module")]
+                SupportedModules::Elf => {
+                    yrx::mods::invoke_dyn::<yrx::mods::ELF>(data)
+                }
+                #[cfg(feature = "pe-module")]
+                SupportedModules::Pe => {
+                    yrx::mods::invoke_dyn::<yrx::mods::PE>(data)
+                }
+                #[cfg(feature = "dotnet-module")]
+                SupportedModules::Dotnet => {
+                    yrx::mods::invoke_dyn::<yrx::mods::Dotnet>(data)
+                }
+                _ => return Ok(py.None().into_bound(py)),
+            };
 
         let module_output = match module_output {
             Some(output) => output,
@@ -303,6 +326,15 @@ impl Module {
     }
 }
 
+/// Returns the names of the supported modules.
+///
+/// These are the modules that can be used in `import` statements in your
+/// rules.
+#[pyfunction]
+fn module_names<'py>(py: Python<'py>) -> PyResult<Bound<'py, PyTuple>> {
+    PyTuple::new(py, yrx::mods::module_names().collect::<Vec<_>>())
+}
+
 /// Compiles a YARA source code producing a set of compiled [`Rules`].
 ///
 /// This function allows compiling simple rules that don't depend on external
@@ -355,7 +387,8 @@ impl Compiler {
     /// The `error_on_slow_pattern` argument tells the compiler to treat slow
     /// patterns as errors, instead of warnings.
     #[new]
-    #[pyo3(signature = (relaxed_re_syntax=false, error_on_slow_pattern=false, includes_enabled=true))]
+    #[pyo3(signature = (relaxed_re_syntax=false, error_on_slow_pattern=false, includes_enabled=true)
+    )]
     fn new(
         relaxed_re_syntax: bool,
         error_on_slow_pattern: bool,
@@ -1067,7 +1100,7 @@ impl JsonDecoder {
         if let Some(encoding) = dict
             .get_item("encoding")?
             .as_ref()
-            .and_then(|encoding| encoding.downcast::<PyString>().ok())
+            .and_then(|encoding| encoding.cast::<PyString>().ok())
         {
             let value = match dict.get_item("value")? {
                 Some(value) => value,
@@ -1076,7 +1109,7 @@ impl JsonDecoder {
 
             if encoding == "base64" {
                 BASE64_STANDARD
-                    .decode(value.downcast::<PyString>()?.to_cow()?.as_bytes())
+                    .decode(value.cast::<PyString>()?.to_cow()?.as_bytes())
                     .expect("decoding base64")
                     .into_bound_py_any(py)
             } else if encoding == "timestamp" {
@@ -1167,6 +1200,7 @@ fn yara_x(m: &Bound<'_, PyModule>) -> PyResult<()> {
     m.add("TimeoutError", m.py().get_type::<TimeoutError>())?;
     m.add("ScanError", m.py().get_type::<ScanError>())?;
     m.add_function(wrap_pyfunction!(compile, m)?)?;
+    m.add_function(wrap_pyfunction!(module_names, m)?)?;
     m.add_class::<Rules>()?;
     m.add_class::<Scanner>()?;
     m.add_class::<ScanResults>()?;
